using System;
using System.Collections;
using Server.Network;
using Server.Engines.Craft;


namespace Server.Items
{        
        public enum FurnitureQuality
	{
		Low,
		Regular,
		Exceptional
	}

	public abstract class BaseFurniture : Item
	{
		

		// Instance values. These values must are unique to each armor piece.
		
                private FurnitureQuality m_Quality;
		private Mobile m_Crafter;
		private CraftResource m_Resource;
		private bool m_Identified, m_PlayerConstructed;
		
		[CommandProperty( AccessLevel.GameMaster )]
		public bool Identified
		{
			get{ return m_Identified; }
			set{ m_Identified = value; InvalidateProperties(); }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public bool PlayerConstructed
		{
			get{ return m_PlayerConstructed; }
			set{ m_PlayerConstructed = value; }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public CraftResource Resource
		{
			get
			{
				return m_Resource;
			}
			set
			{
				if ( m_Resource != value )
				{
					m_Resource = value;
					Hue = CraftResources.GetHue( m_Resource );
				
				}
			}
		}

                [CommandProperty( AccessLevel.GameMaster )]
		public FurnitureQuality Quality
		{
			get{ return m_Quality; }
			set{ m_Quality = value; InvalidateProperties(); }
		}

		
                [CommandProperty( AccessLevel.GameMaster )]
		public Mobile Crafter
		{
			get{ return m_Crafter; }
			set{ m_Crafter = value; InvalidateProperties(); }
		}
		
		public CraftAttributeInfo GetResourceAttrs()
		{
			CraftResourceInfo info = CraftResources.GetInfo( m_Resource );

			if ( info == null )
				return CraftAttributeInfo.Blank;

			return info.AttributeInfo;
		}

		public BaseFurniture( Serial serial ) :  base( serial )
		{
		}

		private static void SetSaveFlag( ref SaveFlag flags, SaveFlag toSet, bool setIf )
		{
			if ( setIf )
				flags |= toSet;
		}

		private static bool GetSaveFlag( SaveFlag flags, SaveFlag toGet )
		{
			return ( (flags & toGet) != 0 );
		}

		[Flags]
		private enum SaveFlag
		{
			None				= 0x00000000,
			Identified			= 0x00000001,
			Crafter				= 0x00000002,
			Resource			= 0x00000004,
			PlayerConstructed	        = 0x00000008,
                        Quality	                        = 0x00000010
		}

                public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 5 ); // version

			SaveFlag flags = SaveFlag.None;

			SetSaveFlag( ref flags, SaveFlag.Identified,		m_Identified != false );
			SetSaveFlag( ref flags, SaveFlag.Crafter,		m_Crafter != null );
			SetSaveFlag( ref flags, SaveFlag.Resource,		m_Resource != DefaultResource );
			SetSaveFlag( ref flags, SaveFlag.PlayerConstructed,	m_PlayerConstructed != false );
                        SetSaveFlag( ref flags, SaveFlag.Quality,		m_Quality != FurnitureQuality.Regular );

			writer.WriteEncodedInt( (int) flags );

			if ( GetSaveFlag( flags, SaveFlag.Crafter ) )
				writer.Write( (Mobile) m_Crafter );

			if ( GetSaveFlag( flags, SaveFlag.Resource ) )
				writer.WriteEncodedInt( (int) m_Resource );

                        if ( GetSaveFlag( flags, SaveFlag.Quality ) )
				writer.WriteEncodedInt( (int) m_Quality );

		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 5:
				case 4:
				case 3:
				{
					SaveFlag flags = (SaveFlag)reader.ReadEncodedInt();

					if ( GetSaveFlag( flags, SaveFlag.Identified ) )
						m_Identified = ( version >= 5 || reader.ReadBool() );

					if ( GetSaveFlag( flags, SaveFlag.Crafter ) )
						m_Crafter = reader.ReadMobile();

					if ( GetSaveFlag( flags, SaveFlag.Resource ) )
						m_Resource = (CraftResource)reader.ReadEncodedInt();
					else
						m_Resource = DefaultResource;

					if ( m_Resource == CraftResource.None )
						m_Resource = DefaultResource;

                                        if ( GetSaveFlag( flags, SaveFlag.Quality ) )
						m_Quality = (FurnitureQuality)reader.ReadEncodedInt();
					else
						m_Quality = FurnitureQuality.Regular;

					if ( version == 3 && m_Quality == FurnitureQuality.Low )
						m_Quality = FurnitureQuality.Regular;

					if ( GetSaveFlag( flags, SaveFlag.PlayerConstructed ) )
						m_PlayerConstructed = true;

					break;
				}
				case 2:
				case 1:
				{
					m_Identified = reader.ReadBool();
					goto case 0;
				}
				case 0:
				{
                                        m_Quality = (FurnitureQuality)reader.ReadInt();
					m_Crafter = reader.ReadMobile();
					
					if ( version >= 2 )
					{
						m_Resource = (CraftResource)reader.ReadInt();
					}
					else
					{
						OreInfo info;

						switch ( reader.ReadInt() )
						{
							default:
							case 0: info = OreInfo.Wood; break;
							case 1: info = OreInfo.Pine; break;
							case 2: info = OreInfo.Walnut; break;
							case 3: info = OreInfo.Maple; break;
							case 4: info = OreInfo.Mahogany; break;
							case 5: info = OreInfo.Ash; break;
							case 6: info = OreInfo.Cedar; break;
							case 7: info = OreInfo.Sycamore; break;
							case 8: info = OreInfo.Cherry; break;
                                                        case 9: info = OreInfo.Yew; break;
						}

						m_Resource = CraftResources.GetFromOreInfo( info );
					}
					break;
				}
			}
			
			if ( version < 5 )
				m_PlayerConstructed = true; // we don't know, so, assume it's crafted
		}

		public virtual CraftResource DefaultResource{ get{ return CraftResource.Wood; } }

		public BaseFurniture( int itemID ) :  base( itemID )
		{
			m_Crafter = null;
                        m_Quality = FurnitureQuality.Regular;
			m_Resource = DefaultResource;
			Hue = CraftResources.GetHue( m_Resource );
			
		}

		private string GetNameString()
		{
			string name = this.Name;

			if ( name == null )
				name = String.Format( "#{0}", LabelNumber );

			return name;
		}

		[Hue, CommandProperty( AccessLevel.GameMaster )]
		public override int Hue
		{
			get{ return base.Hue; }
			set{ base.Hue = value; InvalidateProperties(); }
		}

		public override void AddNameProperty( ObjectPropertyList list )
		{
                        string oreType;

			if ( Hue == 0 )
			{
				oreType = "";
			}
			else
			{
				switch ( m_Resource )
				{
					case CraftResource.Pine:                oreType = "Pine"; break; // pine
                                        case CraftResource.Walnut:                oreType = "Walnut"; break; // walnut
                                        case CraftResource.Maple:                oreType = "maple"; break; // maple
                                        case CraftResource.Mahogany:                oreType = "mahogany"; break; // mahogany
                                        case CraftResource.Ash:                oreType = "ash"; break; // ash
                                        case CraftResource.Cedar:                oreType = "cedar"; break; // cedar
                                        case CraftResource.Sycamore:                oreType = "sycamore"; break; // sycamore
                                        case CraftResource.Cherry:                oreType = "cherry"; break; // cherry
                                        case CraftResource.Yew:                oreType = "yew"; break; // yew
					default: oreType = ""; break;
				}
			}

			if ( m_Quality == FurnitureQuality.Exceptional )
			{
				if ( oreType != "" )
					list.Add( 1053100, "{0}\t{1}", oreType, GetNameString() ); // exceptional ~1_oretype~ ~2_armortype~
				else
					list.Add( 1050040, GetNameString() ); // exceptional ~1_ITEMNAME~
			}
			else
			{
				if ( oreType != "" )
					list.Add( 1053099, "{0}\t{1}", oreType, GetNameString() ); // ~1_oretype~ ~2_armortype~
				else if ( Name == null )
					list.Add( LabelNumber );
				else
					list.Add( Name );
			}
		}

		public override void GetProperties( ObjectPropertyList list )
		{
			base.GetProperties( list );

			if ( m_Crafter != null )
			list.Add( 1050043, m_Crafter.Name ); // crafted by ~1_NAME~

                        if ( m_Quality == FurnitureQuality.Exceptional )
				list.Add( 1060636 ); // exceptional


		}

		public override void OnSingleClick( Mobile from )
		{
			ArrayList attrs = new ArrayList();

			if ( DisplayLootType )
			{
				if ( LootType == LootType.Blessed )
					attrs.Add( new EquipInfoAttribute( 1038021 ) ); // blessed
				else if ( LootType == LootType.Cursed )
					attrs.Add( new EquipInfoAttribute( 1049643 ) ); // cursed
			}

			int number;

			if ( Name == null )
			{
				number = LabelNumber;
			}
			else
			{
				this.LabelTo( from, Name );
				number = 1041000;
			}

			if ( attrs.Count == 0 && Crafter == null && Name != null )
				return;

		}
	}
}
