using System;
using Server.Items;
using Server.Network;

namespace Server.Items
{
	public abstract class BaseLogs : Item, ICommodity
	{
		private CraftResource m_Resource;

		[CommandProperty( AccessLevel.GameMaster )]
		public CraftResource Resource
		{
			get{ return m_Resource; }
			set{ m_Resource = value; InvalidateProperties(); }
		}
		
		string ICommodity.Description
		{
			get
			{
				return String.Format( Amount == 1 ? "{0} {1} log" : "{0} {1} logs", Amount, CraftResources.GetName( m_Resource ).ToLower() );
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 ); // version

			writer.Write( (int) m_Resource );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 1:
				{
					m_Resource = (CraftResource)reader.ReadInt();
					break;
				}
				case 0:
				{
					OreInfo info;

					switch ( reader.ReadInt() )
					{
						case 0: info = OreInfo.Wood; break;
						case 1: info = OreInfo.Pine; break;
						case 2: info = OreInfo.Walnut; break;
						case 3: info = OreInfo.Maple; break;
						case 4: info = OreInfo.Mahogany; break;
						case 5: info = OreInfo.Ash; break;
						case 6: info = OreInfo.Cedar; break;
						case 7: info = OreInfo.Sycamore; break;
						case 8: info = OreInfo.Cherry; break;
                                                case 9: info = OreInfo.Yew; break;
                                                default: info = null; break;
					}

					m_Resource = CraftResources.GetFromOreInfo( info );
					break;
				}
			}
		}

		public BaseLogs( CraftResource resource ) : this( resource, 1 )
		{
		}

		public BaseLogs( CraftResource resource, int amount ) : base( 0x1bdd )
		{
			Stackable = true;
			Weight = 0.1;
			Amount = amount;
			Hue = CraftResources.GetHue( resource );

			m_Resource = resource;
		}

		public BaseLogs( Serial serial ) : base( serial )
		{
		}

		public override void AddNameProperty( ObjectPropertyList list )
		{
			if ( Amount > 1 )
				list.Add( 1050039, "{0}\t#{1}", Amount, 1044466 ); // ~1_NUMBER~ ~2_ITEMNAME~
			else
				list.Add( 1044466 ); // Logs
		}

		public override void GetProperties( ObjectPropertyList list )
		{
			base.GetProperties( list );

			if ( !CraftResources.IsStandard( m_Resource ) )
			{
				int num = CraftResources.GetLocalizationNumber( m_Resource );

				if ( num > 0 )
					list.Add( num );
				else
					list.Add( CraftResources.GetName( m_Resource ) );
			}
		}

		
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class Log : BaseLogs
	{
		[Constructable]
		public Log() : this( 1 )
		{
		}

		[Constructable]
		public Log( int amount ) : base( CraftResource.Wood, amount )
		{
		}

		public Log( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new Log( amount ), amount );
		}
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class PineLog : BaseLogs
	{
		[Constructable]
		public PineLog() : this( 1 )
		{
		}

		[Constructable]
		public PineLog( int amount ) : base( CraftResource.Pine, amount )
		{
		}

		public PineLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new PineLog( amount ), amount );
		}
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class WalnutLog : BaseLogs
	{
		[Constructable]
		public WalnutLog() : this( 1 )
		{
		}

		[Constructable]
		public WalnutLog( int amount ) : base( CraftResource.Walnut, amount )
		{
		}

		public WalnutLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new WalnutLog( amount ), amount );
		}
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class MapleLog : BaseLogs
	{
		[Constructable]
		public MapleLog() : this( 1 )
		{
		}

		[Constructable]
		public MapleLog( int amount ) : base( CraftResource.Maple, amount )
		{
		}

		public MapleLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new MapleLog( amount ), amount );
		}
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class MahoganyLog : BaseLogs
	{
		[Constructable]
		public MahoganyLog() : this( 1 )
		{
		}

		[Constructable]
		public MahoganyLog( int amount ) : base( CraftResource.Mahogany, amount )
		{
		}

		public MahoganyLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new MahoganyLog( amount ), amount );
		}
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class AshLog : BaseLogs
	{
		[Constructable]
		public AshLog() : this( 1 )
		{
		}

		[Constructable]
		public AshLog( int amount ) : base( CraftResource.Ash, amount )
		{
		}

		public AshLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new AshLog( amount ), amount );
		}
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class CedarLog : BaseLogs
	{
		[Constructable]
		public CedarLog() : this( 1 )
		{
		}

		[Constructable]
		public CedarLog( int amount ) : base( CraftResource.Cedar, amount )
		{
		}

		public CedarLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new CedarLog( amount ), amount );
		}
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class SycamoreLog : BaseLogs
	{
		[Constructable]
		public SycamoreLog() : this( 1 )
		{
		}

		[Constructable]
		public SycamoreLog( int amount ) : base( CraftResource.Sycamore, amount )
		{
		}

		public SycamoreLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new SycamoreLog( amount ), amount );
		}
	}

	[FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class CherryLog : BaseLogs
	{
		[Constructable]
		public CherryLog() : this( 1 )
		{
		}

		[Constructable]
		public CherryLog( int amount ) : base( CraftResource.Cherry, amount )
		{
		}

		public CherryLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new CherryLog( amount ), amount );
		}
	}

        [FlipableAttribute( 0x1bdd, 0x1be0 )]
	public class YewLog : BaseLogs
	{
		[Constructable]
		public YewLog() : this( 1 )
		{
		}

		[Constructable]
		public YewLog( int amount ) : base( CraftResource.Yew, amount )
		{
		}

		public YewLog( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new YewLog( amount ), amount );
		}
	}
}
